/*_############################################################################
  _## 
  _##  SNMP4J-Model - SnmpIndexColumn.java  
  _## 
  _##  Copyright (C) 2013-2015  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/
package org.snmp4j.model.snmp.proxy.impl;

import com.snmp4j.smi.SmiIndexInfo;
import com.snmp4j.smi.SmiMaxAccess;
import com.snmp4j.smi.SmiObjectType;
import com.snmp4j.smi.SmiSyntax;
import org.snmp4j.model.snmp.proxy.SnmpProxyIndexColumn;
import org.snmp4j.smi.OID;

/**
 * The SnmpIndexColumn represents a columnar object that is used as an INDEX element of a specific table.
 * An instance of this class can only be used with the table it has been created for, because the
 * {@link com.snmp4j.smi.SmiIndexInfo} may vary from table to table.
 *
 * @param <T>
 *   specifies the value type of the index column.
 *
 * @author Frank Fock
 */
public class SnmpIndexColumn<T> extends SnmpColumn<T> implements SnmpProxyIndexColumn<T> {

  private SmiIndexInfo smiIndexInfo;

  /**
   * Creates a index column.
   * @param oid
   *    the OID of the columnar object that represents this (sub-)index column.
   * @param valueClass
   *    the value class of the index column's value.
   * @param smiIndexInfo
   *    the description of the index
   * @param smiSyntax
   *    the SMI syntax of the index column.
   */
  public SnmpIndexColumn(OID oid, Class<T> valueClass, SmiIndexInfo smiIndexInfo, SmiSyntax smiSyntax) {
    super(oid, valueClass, SmiMaxAccess.notAccessible, smiSyntax);
    this.smiIndexInfo = smiIndexInfo;
  }

  /**
   * Creates a index column.
   * @param oid
   *    the OID of the columnar object that represents this (sub-)index column.
   * @param valueClass
   *    the value class of the index column's value.
   * @param smiSyntax
   *    the SMI syntax of the index column.
   * @param minWidth
   *    the minimum width in characters of the values for this column. This servers as a hint
   *    for the column size calculation when this model is being rendered for display.
   * @param maxWidth
   *    the maximum width in characters of the values of this column. This servers as a hint
   *    for the column size calculation when this model is being rendered for display.
   * @param smiIndexInfo
   *    the description of the index
   */
  public SnmpIndexColumn(OID oid, Class<T> valueClass, SmiSyntax smiSyntax, int minWidth, int maxWidth,
                         SmiIndexInfo smiIndexInfo) {
    super(oid, valueClass, SmiMaxAccess.notAccessible, smiSyntax, minWidth, maxWidth);
    this.smiIndexInfo = smiIndexInfo;
  }

  /**
   * Creates a index column.
   * @param oid
   *    the OID of the columnar object that represents this (sub-)index column.
   * @param valueClass
   *    the value class of the index column's value.
   * @param smiSyntax
   *    the SMI syntax of the index column.
   * @param minWidth
   *    the minimum width in characters of the values for this column. This servers as a hint
   *    for the column size calculation when this model is being rendered for display.
   * @param maxWidth
   *    the maximum width in characters of the values of this column. This servers as a hint
   *    for the column size calculation when this model is being rendered for display.
   * @param header
   *    the header string for this column (this value is not actually used by the proxy but can be used
   *    by a column renderer.
   * @param smiIndexInfo
   *    the description of the index.
   */
  public SnmpIndexColumn(OID oid, Class<T> valueClass, SmiSyntax smiSyntax,
                         int minWidth, int maxWidth, String header, SmiIndexInfo smiIndexInfo) {
    super(oid, valueClass, SmiMaxAccess.notAccessible, smiSyntax, minWidth, maxWidth, header);
    this.smiIndexInfo = smiIndexInfo;
  }

  @Override
  public boolean hasImpliedLength() {
    return smiIndexInfo.hasImpliedLength();
  }

  @Override
  public int getMinLength() {
    return smiIndexInfo.getMinLength();
  }

  @Override
  public int getMaxLength() {
    return smiIndexInfo.getMaxLength();
  }

  @Override
  public int getSnmpSyntax() {
    return smiIndexInfo.getSnmpSyntax();
  }

  @Override
  public SmiObjectType getTableEntry() {
    return smiIndexInfo.getTableEntry();
  }
}
